#include "xmlmodel.h"

XmlModel::XmlModel(QObject *parent):QAbstractItemModel(parent)
{
    rootItem=new QObject(this);
}

XmlModel::~XmlModel()
{
    delete rootItem;
}

QObject* XmlModel::getRoot() const
{
    return rootItem;
}

QModelIndex XmlModel::getIndex() const
{
    return indexOfObject;
}

QObject* XmlModel::objectByIndex(const QModelIndex &index) const{
    if(!index.isValid())
        return rootItem;
    return static_cast<QObject*>(index.internalPointer());
}

QVariant XmlModel::data(const QModelIndex &index, int role) const
{
    if (role==Qt::DisplayRole) {
        return objectByIndex(index)->property("name");
    }
    if (role==Qt::FontRole) {
        if (objectByIndex(index)->property("font")==true) {
            QFont font;
            font.setBold(true);
            return font;
        }
    }
    return QVariant();
}

Qt::ItemFlags XmlModel::flags(const QModelIndex &index) const
{
    if (!index.isValid()) {
        return Qt::NoItemFlags;
    }

    return QAbstractItemModel::flags(index);
}

QModelIndex XmlModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent)) {
        return QModelIndex();
    }

    QObject* parentObj=objectByIndex(parent);
    return createIndex(row,column,parentObj->children().at(row));
}

QModelIndex XmlModel::parent(const QModelIndex &child) const
{
    QObject* childObject=objectByIndex(child);
    QObject* parentObject=childObject->parent();

    if(!child.isValid()){
        return QModelIndex();
    }
    if(parentObject==rootItem){
        return QModelIndex();
    }
    return createIndex(parentObject->parent()->children().indexOf(parentObject),0, parentObject);
}

int XmlModel::rowCount(const QModelIndex &parent) const
{
    if (parent.column() > 0) {
        return 0;
    }

    if (!parent.isValid()) {
        return rootItem->children().count();
    } else {
        return static_cast<QObject*>(parent.internalPointer())->children().count();
    }
    return rootItem->children().count();
}

int XmlModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 1;
}

void XmlModel::setIndex(QModelIndex index)
{
    indexOfObject=index;
}

void XmlModel::makeActive()
{
    QObject* curObject=objectByIndex(indexOfObject);
    activeObject->setProperty("font",false);
    curObject->setProperty("font",true);
    activeObject=curObject;
}

void addElement(const QDomNode& node,QObject*parent) {
    if (!(node.isNull())) {
       if ((!(node.isText()))&&(node.isElement())) {
            QObject* newNode=new QObject(parent);
            newNode->setProperty("name",node.toElement().tagName());
            if (node.hasChildNodes() ){
                QDomNodeList ListNodes=node.childNodes();
                for (int i=0;i<ListNodes.count();++i) {
                    addElement(ListNodes.at(i),newNode);
                }
            }
        }
        if (node.isText()) {
            QObject* newNode=new QObject(parent);
            newNode->setProperty("name",node.toText().data());
        }
    }
}

void XmlModel::addFiles(QStringList filesName)
{
    for(const auto& path:filesName){
        QDomDocument document;
        QFile file(path);
        if (file.open(QIODevice::ReadOnly)){
            if (document.setContent(&file)) {
                QFileInfo fileInfo(path);
                QString nameOfCurFile=fileInfo.fileName();
                bool flag=true;
                for(const auto obj:rootItem->children()) {
                    if (nameOfCurFile==obj->property("name").toString()) {
                        flag=false;
                    }
                }
                if(flag){
                    QObject* FirstElem=new QObject(rootItem);
                    FirstElem->setProperty("name",nameOfCurFile);
                    QDomElement element = document.documentElement();
                    addElement(element,FirstElem);
                } else {
                    QMessageBox message(QMessageBox::Critical, tr("Ошибка парсинга"),
                                                tr("XML-документ ") + nameOfCurFile + tr(" уже открыт"), QMessageBox::Ok);
                    message.exec();
                }
            } else {
                QMessageBox message(QMessageBox::Critical, tr("Ошибка парсинга"),
                                            tr("Данные из ") + path + tr(" не загружаются"), QMessageBox::Ok);
                message.exec();
            }
        } else {
            QMessageBox message(QMessageBox::Critical, tr("Ошибка парсинга"),
                                        tr("XML-документ ") + path + tr(" не открывается"), QMessageBox::Ok);
            message.exec();
        }
        file.close();
    }
    if (activeObject==Q_NULLPTR) {
        objectByIndex(index(0,0))->setProperty("font",true);
        activeObject=objectByIndex(index(0,0));
    }
}
